# 1 简介
该工具分为三个主要进程
- 客户端（VideoClientHost）：单路视频流云手机客户端实例，可以连接一路视频流云手机；在服务端进程存在的情况下，会将统计数据上报至服务端进程。
- 服务端（VideoEmulatorServer）：接收当前服务器上所有客户端上报的统计信息，并提供给控制端进行分析、显示和存档。
- 控制端（cmdline）：用于控制服务端，可以接收服务端发送的统计信息，并进行分析、显示和存档

```
    ServerA:8001 | <- VideoClientHost -> |                     |
    ServerA:8002 | <- VideoClientHost -> |                     | -> cmdline[ServerA]
    ServerB:8001 | <- VideoClientHost -> | VideoEmulatorServer |
       ......    |         ......        |                     | -> cmdline[ServerB]
    ServerB:800N | <- VideoClientHost -> |                     |
```

# 2 使用方法
- 前置需求：已安装numactl
- 如果需要在云手机服务端使用该工具，请在config中修改绑核，并使用127.0.0.1访问本地云手机
## 命令说明
- 启动并连接一个或多个视频流客户端进程，可以在config中配置客户端绑核
```
./test.sh connect $ip $start_port $end_port
./test.sh connect $ip $port
```

- 断开一个或多个客户端进程
```
./test.sh disconnect $ip $start_port $end_port
./test.sh disconnect $ip $port
```

- 断开当前所有客户端
```
./test.sh disconnect all
```

- 打印指定客户端帧率和延时情况（可以在config里指定显示格式）
```
./test.sh display $ip $start_port $end_port
```

- 开始收集指定客户端帧率和延时情况（可以在config里指定显示格式）
```
./test.sh start $ip $start_port $end_port $run_time （单位：秒；最大值3小时）
```

- 查询当前已连接的客户端
```
./test.sh query
```

- 清理所有客户端进程和服务端进程（建议每完成一次测试后都执行，避免僵尸进程）
```
./test.sh clean
```

## 配置文件说明
## 显示配置
- FPS_BASE：基础帧率，支持30或60，同时决定了统计时使用的低帧率标准
- INSTANCE_IN_ROW：每行显示多少个客户端统计信息
- COLLAPSE_NULL_CLIENT：是否折叠未连接的端口，0: 不折叠，未连接端口显示为null; 1:折叠，未连接端口不显示
- NUMA_NUM：显示最终结果时，分几个NUMA进行统计（仅在按NUMA均衡方式启动云手机时有效）
## 绑核配置
- CPU_FOR_CLIENT：客户端（VideoClientHost）绑核，连接时会循环使用该配置中提供的CPU核
- CPU_FOR_SERVER：提供给服务端（VideoEmulatorServer）使用的CPU核
- CPU_FOR_CMD：提供给控制端（cmdline）使用的CPU核

# 3 构建方法
该工具需要构建四个组件，依赖VMIEngine仓和VMIEngine-Demo仓提供源码，要求VMIEngine仓和VMIEngine-Demo仓在同一目录下
```
|-- VMIEngine
|-- VMIEngine-Demo
```
- VideoClientHost：依赖VMIEngine仓提供视频流客户端源码
- VideoEmulatorServer：无额外依赖
- cmdline：无额外依赖
- libCommunication.so：依赖VMIEngine-Demo仓提供网络模块源码

## 构建命令
```
./build.sh build  #全量编译
./build.sh inc    #增量编译
./build.sh clean  #清理
```
构建成功后，在output目录生成对应的文件
